
' -----------------------------------------------------------------------
' <copyright file="BroadcastProgram.cs" company="Microsoft">
' Copyright © Microsoft Corporation.  All rights reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
' FITNESS FOR A PARTICULAR PURPOSE.
' </copyright>
'  <notes>
'                  \\|//
'                  (o o)
' --------------ooO-(_)-Ooo--------------------
'  Authors: Developer Product Evangelist Team (greg.prentice)
'  License: Apache License Version 2.0, January 2004
'                       Oooo
' --------------oooO----(  )-------------------
'               (  )    (_/
'                \_)
'  </notes>
' -----------------------------------------------------------------------

Imports System.Linq
Imports System.Threading
Imports System.Threading.Tasks
Imports Microsoft.WindowsAzure.MediaServices.Client
Namespace LiveMediaManager

	''' <summary>
	''' Defines a wrapper class for interacting with a Media Services IProgram interface.
	''' </summary>
	Public Class BroadcastProgram
        Implements IBroadcastProgram

        ''' <summary>
        ''' Initializes a new instance of the <see cref="BroadcastProgram"/> class.
        ''' </summary>
        ''' <param name="program">the Media Service program</param>
        ''' <param name="channelContext">the channel context that owns the program</param>
        Public Sub New(program As IProgram, channelContext As ChannelContext)
            If program Is Nothing Then
                Throw New ArgumentNullException("program")
            End If

            If channelContext Is Nothing Then
                Throw New ArgumentNullException("channelContext")
            End If

            m_Program = program
            m_StartDateTime = DateTime.Now
            Me.ChannelContext = channelContext
        End Sub

        ''' <summary>
        ''' Gets a reference to the owning channel.
        ''' </summary>
        Public Property ChannelContext() As ChannelContext
            Get
                Return m_ChannelContext
            End Get
            Private Set(value As ChannelContext)
                m_ChannelContext = value
            End Set
        End Property
        Private m_ChannelContext As ChannelContext

        ''' <summary>
        ''' Gets a reference to the program interface.
        ''' </summary>
        Public ReadOnly Property Program() As IProgram Implements IBroadcastProgram.Program
            Get
                Return m_Program
            End Get
        End Property
        Private m_Program As IProgram

        ''' <summary>
        ''' Gets the requested start date and time of this program.
        ''' Note: exercise for the developer to utilize this value.
        ''' </summary>
        Public ReadOnly Property StartDateTime() As DateTime Implements IBroadcastProgram.StartDateTime
            Get
                Return m_StartDateTime
            End Get
        End Property
        Private m_StartDateTime As DateTime

        ''' <summary>
        ''' Attempts to create a program with the given configuration settings.
        ''' </summary>
        ''' <param name="channel">defines the channel</param>
        ''' <param name="settings">Configuration of the program</param>
        ''' <returns>a task used for tracking the completion of the operation</returns>
        Public Shared Function Create(channel As ChannelContext, settings As ProgramCreationOptions) As Task(Of BroadcastProgram)
            If channel Is Nothing Then
                Throw New ArgumentNullException("channel")
            End If

            If settings Is Nothing OrElse String.IsNullOrEmpty(settings.Name) Then
                Throw New ArgumentNullException("settings")
            End If

            Dim program As IBroadcastProgram = channel.BroadcastSchedule.Find(settings.Name)

            If program IsNot Nothing Then
                Throw New ArgumentException("Program name already exists for the selected channel", "name")
            End If

            Dim action As Func(Of BroadcastProgram) = Function()
                                                          Dim asset As IAsset = channel.MediaContext.LiveServices.Assets.Create(settings.Name + "_asset", AssetCreationOptions.None)

                                                          Dim p As Task(Of IProgram) = channel.Channel.Programs.CreateAsync(settings)
                                                          p.Wait()
                                                          Return New BroadcastProgram(p.Result, channel)

                                                      End Function

            Dim task As New Task(Of BroadcastProgram)(action)
            task.Start()
            Return task
        End Function

        ''' <summary>
        ''' Updates a programs current state.
        ''' </summary>
        Public Sub Refresh()
            If Me.Program Is Nothing Then
                Throw New ArgumentNullException("name")
            End If

            ' Refresh the channel first.
            Me.ChannelContext.Refresh()

            m_Program = (From p In Me.ChannelContext.Channel.Programs Where p.Name = Me.Program.Name).FirstOrDefault()

            If Me.Program Is Nothing Then
                Throw New ArgumentNullException("name")
            End If
        End Sub

        ''' <summary>
        ''' Starts the current program which basically begins archiving the current live stream into
        ''' an IAsset in blob storage.
        ''' </summary>
        ''' <returns>a task used for tracking the completion of the operation</returns>
        Public Function Start() As Task Implements IBroadcastProgram.Start
            Me.Refresh()

            If Me.Program.Channel.State <> ChannelState.Running Then
                Throw New ArgumentOutOfRangeException("program", "The program's channel must be in a Running State before starting a program!")
            End If

            Dim action As Action = Nothing

            Select Case Me.Program.State
                Case ProgramState.Running, ProgramState.Starting
                    action = Function()

                             End Function
                    Exit Select
                Case ProgramState.Stopped
                    action = Function()
                                 Me.Program.Start()
                                 Refresh()

                             End Function
                    Exit Select
                Case Else
                    Throw New ArgumentOutOfRangeException("program", "The program is not in the proper state!")
            End Select

            Dim task As New Task(action)
            task.Start()
            Return task
        End Function

        ''' <summary>
        ''' Stops the current program which basically stops archiving the current live stream into
        ''' an IAsset in blob storage.
        ''' </summary>
        ''' <returns>a task used for tracking the completion of the operation</returns>
        Public Function [Stop]() As Task Implements IBroadcastProgram.Stop
            Me.Refresh()

            Dim action As Action = Nothing

            Select Case Me.Program.State
                Case ProgramState.Starting
                    action = Function()
                                 WaitForState(ProgramState.Running)
                                 Me.Program.StopAsync().Wait()

                             End Function
                    Exit Select
                Case ProgramState.Running
                    action = Function()
                                 Me.Program.StopAsync().Wait()

                             End Function
                    Exit Select
                Case ProgramState.Stopping
                    action = Function()
                                 WaitForState(ProgramState.Stopped)

                             End Function
                    Exit Select
                Case ProgramState.Stopped
                    action = Function()

                             End Function
                    Exit Select
                Case Else
                    Throw New NotSupportedException("Unknown program state found while attempting to stop program.")
            End Select

            Dim task As New Task(action)
            task.Start()
            Return task
        End Function

        ''' <summary>
        ''' Checks that a task in is a stopped state, if true then deletes it from the system.
        ''' Note we do not delete any associated assets as it is assumed they are being kept
        ''' for archival.
        ''' </summary>
        ''' <param name="all">if true then remove assets and locators as well</param>
        ''' <returns>a task used for tracking the completion of the operation</returns>
        Public Function Delete(all As Boolean) As Task Implements IBroadcastProgram.Delete
            Me.Refresh()

            If Me.Program.State <> ProgramState.Stopped Then
                Throw New ArgumentOutOfRangeException("program", "The program must be in a stopped State before being deleted!")
            End If

            Dim action As Action = Function()
                                       Try
                                           ' Get the the asset
                                           Dim asset As IAsset = Me.ChannelContext.MediaContext.LiveServices.FindAsset(Me.Program)
                                           Me.Program.DeleteAsync().Wait()
                                           m_Program = Nothing
                                           Me.ChannelContext = Nothing

                                           If asset IsNot Nothing AndAlso all Then
                                               ' You have to delete the assets after the program has been deleted, else
                                               '  an exception is thrown.
                                               asset.DeleteAsync().Wait()
                                           End If
                                       Catch e As Exception
                                           System.Diagnostics.Debug.WriteLine(e.Message)

                                       End Try

                                   End Function

            Dim task As New Task(action)
            task.Start()
            Return task
        End Function

        ''' <summary>
        ''' Waits for the program to transition into the requested state.
        ''' </summary>
        ''' <param name="state">the requested state</param>
        Public Function WaitForState(state As ProgramState) As Task(Of Boolean) Implements IBroadcastProgram.WaitForState
            Dim action As Func(Of Boolean) = Function()
                                                 Dim retries As Integer = Me.ChannelContext.MediaContext.Retries

                                                 While True
                                                     Me.Refresh()

                                                     If Me.Program.State = state Then
                                                         Return True
                                                     End If

                                                     retries -= 1

                                                     If retries = 0 Then
                                                         Exit While
                                                     End If

                                                     Thread.Sleep(Me.ChannelContext.MediaContext.SleepMilliseconds)
                                                 End While

                                                 Return False

                                             End Function

            Dim task As New Task(Of Boolean)(action)
            task.Start()
            Return task
        End Function

        
    End Class
End Namespace
