
' ----------------------------------------------------------------------
' <copyright file="BroadcastSchedule.cs" company="Microsoft">
' Copyright © Microsoft Corporation.  All rights reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
' FITNESS FOR A PARTICULAR PURPOSE.
' </copyright>
'  <notes>
'                  \\|//
'                  (o o)
' --------------ooO-(_)-Ooo--------------------
'  Authors: Developer Product Evangelist Team (greg.prentice)
'  License: Apache License Version 2.0, January 2004
'                       Oooo
' --------------oooO----(  )-------------------
'               (  )    (_/
'                \_)
'  </notes>
' -----------------------------------------------------------------------

Imports System.Collections.Generic
Imports System.Linq
Imports System.Threading.Tasks
Imports Microsoft.WindowsAzure.MediaServices.Client
Namespace LiveMediaManager

	''' <summary>
	''' Defines a class that represents a list programs which represent a dated and time gated
	''' streaming assets. 
	''' </summary>
	Public Class BroadcastSchedule
		Inherits List(Of IBroadcastProgram)
        Implements IBroadcastSchedule
        ''' <summary>
        ''' Initializes a new instance of the <see cref="BroadcastSchedule"/> class.
        ''' </summary>
        ''' <param name="channel">channel associated with this schedule</param>
        Public Sub New(channel As ChannelContext)
            If channel Is Nothing Then
                Throw New ArgumentNullException("channel")
            End If

            Me.Channel = channel

            Me.Refresh()
        End Sub

        ''' <summary>
        ''' Gets the reference to the channel.
        ''' </summary>
        Public Property Channel() As ChannelContext
            Get
                Return m_Channel
            End Get
            Private Set(value As ChannelContext)
                m_Channel = Value
            End Set
        End Property
        Private m_Channel As ChannelContext

        ''' <summary>
        ''' Updates the status of our programs.
        ''' </summary>
        Public Sub Refresh()
            Me.Channel.Refresh()

            Me.Clear()

            For Each program As IProgram In Me.Channel.Channel.Programs
                ' TODO: load broadcast program from a data strore, should include
                '  start stop dates / times.
                Dim broadcastProgram As IBroadcastProgram = New BroadcastProgram(program, Me.Channel)
                Me.Add(broadcastProgram)
            Next
        End Sub

        ''' <summary>
        ''' Creates a program associated with this schedule.
        ''' </summary>
        ''' <param name="options">settings used during the creation of the program</param>
        ''' <returns>a task used for tracking the completion of the operation</returns>
        Public Function Create(options As ProgramCreationOptions) As Task(Of IBroadcastProgram) Implements IBroadcastSchedule.Create
            If options Is Nothing Then
                Throw New ArgumentNullException("settings")
            End If

            Dim func As Func(Of IBroadcastProgram) = Function()
                                                         Dim t As Task(Of BroadcastProgram) = BroadcastProgram.Create(Me.Channel, options)
                                                         t.Wait()
                                                         Me.Add(t.Result)
                                                         Return t.Result

                                                     End Function

            Dim task As New Task(Of IBroadcastProgram)(func)
            task.Start()
            Return task
        End Function

        ''' <summary>
        ''' Locates a program.
        ''' </summary>
        ''' <param name="name">name of program to locate</param>
        ''' <returns>the program or null if not found</returns>
        Public Function Find(name As String) As IBroadcastProgram Implements IBroadcastSchedule.Find
            If String.IsNullOrEmpty(name) Then
                Throw New ArgumentNullException("name")
            End If

            Dim program As IBroadcastProgram

            ' You really need to refresh your internal current list after this call.
            Me.Channel.Refresh()

            program = (From x In Me Where x.Program.Name = name).FirstOrDefault()

            Return program
        End Function

        ''' <summary>
        ''' Stops all currently running programs associated with this schedule.
        ''' </summary>
        ''' <returns>a task used for tracking the completion of the operation</returns>
        Public Function StopAllPrograms() As List(Of Task) Implements IBroadcastSchedule.StopAllPrograms
            Me.Refresh()

            Dim list As New List(Of Task)()
            For Each program As IBroadcastProgram In Me
                list.Add(program.[Stop]())
            Next

            Return list
        End Function

        ''' <summary>
        ''' Attempts to delete all of the programs associated with this schedule.
        ''' </summary>
        ''' <returns>a task used for tracking the completion of the operation</returns>
        Public Function DeleteAllPrograms(Optional deleteAssets As Boolean = False) As List(Of Task) Implements IBroadcastSchedule.DeleteAllPrograms
            Me.Refresh()

            Dim list As New List(Of Task)()

            For Each program As IBroadcastProgram In Me
                list.Add(program.Delete(deleteAssets))
            Next

            Return list
        End Function

        Public Function Reset() As Task
            Dim func As Func(Of Boolean) = Function()
                                                        Dim tasks As Task() = Me.StopAllPrograms().ToArray()
                                                        Task.WaitAll(tasks)

                                                        Me.Channel.WaitForState(ChannelState.Running)

                                                        Return False

                                                    End Function

            Dim task__1 As New Task(Of Boolean)(func)
            task__1.Start()
            Return task__1


        End Function

        Public Function WaitForAllPrograms(state As ProgramState) As List(Of Task(Of Boolean)) Implements IBroadcastSchedule.WaitForAllPrograms
            Me.Refresh()

            Dim list As New List(Of Task(Of Boolean))()

            For Each program As IBroadcastProgram In Me
                list.Add(program.WaitForState(ProgramState.Stopped))
            Next

            Return list
        End Function
    End Class
End Namespace
