
' -----------------------------------------------------------------------
' <copyright file="ChannelContext.cs" company="Microsoft">
' Copyright © Microsoft Corporation.  All rights reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
' FITNESS FOR A PARTICULAR PURPOSE.
' </copyright>
'  <notes>
'                  \\|//
'                  (o o)
' --------------ooO-(_)-Ooo--------------------
'  Authors: Developer Product Evangelist Team (greg.prentice)
'  License: Apache License Version 2.0, January 2004
'                       Oooo
' --------------oooO----(  )-------------------
'               (  )    (_/
'                \_)
'  </notes>
' -----------------------------------------------------------------------

Imports System.Collections.Generic
Imports System.Linq
Imports System.Threading
Imports System.Threading.Tasks
Imports Microsoft.WindowsAzure.MediaServices.Client
Namespace LiveMediaManager

	''' <summary>
	''' A class that contains all of the management aspects as related to the IChannel interfaces.
	''' Takes care of ensuring that the most up-to-date IChannel is loaded since other systems
	''' could potentially interfere with operations. Example someone could delete the channel out from under
	''' the this class's current context.
	''' </summary>
	Public Class ChannelContext
		Implements IDisposable
		''' <summary>
		''' Initializes a new instance of the <see cref="ChannelContext"/> class.
		''' </summary>
		''' <param name="channel">a reference to the Azure Media Services channel</param>
		''' <param name="mediaContext">a the connection context associated with this channel</param>
		Public Sub New(channel As IChannel, streamEndpointName As String, mediaContext As MediaContext)
			If channel Is Nothing Then
				Throw New ArgumentNullException("channel")
			End If

			If String.IsNullOrEmpty(streamEndpointName) Then
				Throw New ArgumentNullException("streamEndpointName")
			End If

			If mediaContext Is Nothing Then
				Throw New ArgumentNullException("mediaContext")
			End If

			Me.Channel = channel
			Me.MediaContext = mediaContext

			' This should be loaded from a data store.
			Me.BroadcastSchedule = New BroadcastSchedule(Me)

			Me.StreamEndpointName = streamEndpointName
		End Sub

		''' <summary>
		''' Gets a list of programs or IProgram).
		''' </summary>
		Public Property BroadcastSchedule() As IBroadcastSchedule
			Get
				Return m_BroadcastSchedule
			End Get
			Private Set
				m_BroadcastSchedule = Value
			End Set
		End Property
		Private m_BroadcastSchedule As IBroadcastSchedule

		''' <summary>
		''' Gets a reference to the Azure Media Services connection context.
		''' </summary>
		Public Property MediaContext() As MediaContext
			Get
				Return m_MediaContext
			End Get
			Private Set
				m_MediaContext = Value
			End Set
		End Property
		Private m_MediaContext As MediaContext

		''' <summary>
		''' Gets the streaming endpoint server name used with this channel.
		''' </summary>
		Public Property StreamEndpointName() As String
			Get
				Return m_StreamEndpointName
			End Get
			Private Set
				m_StreamEndpointName = Value
			End Set
		End Property
		Private m_StreamEndpointName As String

		''' <summary>
		''' Gets a reference to the Azure Media Services Channel.
		''' </summary>
		Friend Property Channel() As IChannel
			Get
				Return m_Channel
			End Get
			Private Set
				m_Channel = Value
			End Set
		End Property
		Private m_Channel As IChannel

		''' <summary>
		''' Updates the member Channel variable with the most recent state.
		''' </summary>
		''' <exception cref="ArgumentNullException">If at any point prior or post the refresh we find that this.Channel is null</exception>
		Public Sub Refresh()
			If Me.Channel Is Nothing Then
				Throw New ArgumentNullException("name")
			End If

            Me.Channel = (From x In Me.MediaContext.LiveServices.Channels Where x.Name = Me.Channel.Name).FirstOrDefault()

			If Me.Channel Is Nothing Then
				Throw New ArgumentNullException("name")
			End If
		End Sub

		''' <summary>
		''' Deletes the channel.
		''' </summary>
		''' <exception cref="ArgumentException">If the channel is not stopped</exception>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Public Function Delete() As Task
			Me.Refresh()

			If Me.Channel.State <> ChannelState.Stopped Then
				Throw New ArgumentException("A channel must be stopped in order to be deleted.")
			End If

			Dim action As Action(Of Object) = Function(cc As Object) 
			Dim channel As ChannelContext = TryCast(cc, ChannelContext)
			channel.DeletePrograms().Wait()
			channel.Channel.DeleteAsync().Wait()
			channel.Channel = Nothing

End Function

			Dim task As New Task(action, DirectCast(Me, Object))
			task.Start()
			Return task
		End Function

		''' <summary>
		''' Starts the channel.
		''' </summary>
		''' <exception cref="ArgumentException">If the channel is not stopped</exception>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Public Function Start() As Task
			Me.Refresh()

			If Me.Channel.State <> ChannelState.Stopped Then
				Throw New ArgumentOutOfRangeException("name", "The channel must be in a stopped state!")
			End If

			Return Me.Channel.StartAsync()
		End Function

		''' <summary>
		''' Stops the channel by first checking the current state, then shutting down any running programs and finally stopping the channel.
		''' </summary>
		''' <exception cref="NotSupportedException">If an unknown channel state is found</exception>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Public Function [Stop]() As Task
			' Make sure you get the most recent channel info before our test.
			Me.Refresh()

			Dim action As Action(Of Object) = Nothing

			Select Case Me.Channel.State
				Case ChannelState.Starting
					action = Function(cc) 
					Dim channel As ChannelContext = TryCast(cc, ChannelContext)
					channel.WaitForState(ChannelState.Running)
					Dim list As List(Of Task) = channel.BroadcastSchedule.StopAllPrograms()
					list.Add(channel.Channel.StopAsync())
					Task.WaitAll(list.ToArray())

End Function
					Exit Select
				Case ChannelState.Running
					action = Function(cc) 
					Dim channel As ChannelContext = TryCast(cc, ChannelContext)
					Dim list As List(Of Task) = Me.BroadcastSchedule.StopAllPrograms()
					list.Add(Me.Channel.StopAsync())
					Task.WaitAll(list.ToArray())

End Function
					Exit Select
				Case ChannelState.Stopping
					action = Function(cc) 
					Dim channel As ChannelContext = TryCast(cc, ChannelContext)
					channel.WaitForState(ChannelState.Stopped)

End Function
					Exit Select
				Case ChannelState.Stopped
					action = Function(cc) 

End Function
					Exit Select
				Case Else
					Throw New NotSupportedException("Unknowk channel state found while attempting to stop the channel.")
			End Select

			Dim task__1 As New Task(action, DirectCast(Me, Object))
			task__1.Start()
			Return task__1
		End Function

		''' <summary>
		''' Will attempt to block until the requested channel state transition occurs.  Will block until either
		''' the state transition occurs or we have waited too long throwing an exception.  Won't block forever.
		''' </summary>
		''' <param name="state">the requested channel state</param>
		''' <exception cref="TimeoutException">If the maximum number of retries have been reached</exception>
		Public Sub WaitForState(state As ChannelState)
			Dim retries As Integer = Me.MediaContext.Retries

			While True
				Me.Refresh()

				If Me.Channel.State = state Then
					Return
				End If

				retries -= 1

				If retries = 0 Then
					Throw New TimeoutException("Attempting to shutdown programs have exceeded the alotted retries.")
				End If

				Thread.Sleep(Me.MediaContext.SleepMilliseconds)
			End While
		End Sub


		''' <summary>
		''' Deletes all of the programs associated with this channel.
		''' </summary>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Public Function DeletePrograms() As Task
			Me.Refresh()

			If Me.Channel.State <> ChannelState.Stopped Then
				Throw New ArgumentException("A channel must be stopped in order to be deleted.")
			End If

			Dim action As Action = Function() 
			Dim list As List(Of Task) = Me.BroadcastSchedule.StopAllPrograms()
			Task.WaitAll(list.ToArray())
			list = Me.BroadcastSchedule.DeleteAllPrograms()
			Task.WaitAll(list.ToArray())
			Me.BroadcastSchedule.Clear()

End Function

			Dim task__1 As New Task(action)
			task__1.Start()
			Return task__1
		End Function

		''' <summary>
		''' Does nothing today but close down context
		''' </summary>
        Public Sub Dispose() Implements IDisposable.Dispose
            Me.Channel = Nothing
            Me.MediaContext = Nothing
        End Sub

		''' <summary>
		''' Determines if an streaming endpoint exists, creates one if it does not, then ensures
		''' the service has started.
		''' </summary>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Public Function CreateStreamingEndpointAndStart() As Task
			Dim action As Action = Function() 
			Dim streamEndpoint As IStreamingEndpoint = Me.MediaContext.LiveServices.FindStreamingEndpoint(Me.StreamEndpointName)

			If streamEndpoint Is Nothing Then
				streamEndpoint = Me.MediaContext.LiveServices.StreamingEndpoints.Create(Me.StreamEndpointName, 2)
			End If

			If streamEndpoint.State = StreamingEndpointState.Stopped Then
				streamEndpoint.StartAsync().Wait()
			End If

End Function

			Dim task As New Task(action)
			task.Start()
			Return task
		End Function

		''' <summary>
		''' Locates the asset associated with the program and attempts to create 
		''' a locator and associated list of URI.
		''' </summary>
		''' <param name="program">reference to the program owning the asset.</param>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Public Function CreateLocators(program As IProgram) As Task(Of IDictionary(Of VideoFormat, String))
			Dim func As Func(Of IDictionary(Of VideoFormat, String)) = Function() 
			Dim list As IDictionary(Of VideoFormat, String) = Nothing
			Dim asset As IAsset = Me.MediaContext.LiveServices.FindAsset(program)
			If asset IsNot Nothing Then
				Me.MediaContext.LiveServices.CreateLocators(asset, TimeSpan.FromDays(30))
				list = Me.MediaContext.LiveServices.CreateStreamingURIs(asset)
			End If

			Return list

End Function

			Dim task As New Task(Of IDictionary(Of VideoFormat, String))(func)
			task.Start()
			Return task
		End Function

		''' <summary>
		''' Deletes a program's assets locators.
		''' </summary>
		''' <param name="program">reference to the program owning the asset.</param>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Public Function DeleteLocators(program As IProgram) As Task
			Dim action As Action = Function() 
			Dim asset As IAsset = Me.MediaContext.LiveServices.FindAsset(program)
			If asset IsNot Nothing Then
				For Each locator As ILocator In asset.Locators
					locator.Delete()
				Next
			End If

End Function

			Dim task As New Task(action)
			task.Start()
			Return task
		End Function

		''' <summary>
		''' Attempts to shutdown the streaming endpoint servers and then deletes the instances.
		''' </summary>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Public Function StopAndDeleteStreamingEndpoints() As Task
			Dim action As Action = Function() 
			Dim streamEndpoint As IStreamingEndpoint = Me.MediaContext.LiveServices.FindStreamingEndpoint(Me.StreamEndpointName)

			If streamEndpoint IsNot Nothing Then
				streamEndpoint.StopAsync().Wait()
				streamEndpoint.DeleteAsync().Wait()
			End If

End Function

			Dim task As New Task(action)
			task.Start()
			Return task
		End Function

		''' <summary>
		''' Used to add or remove streaming endpoint reserved units.  By default you always get an extra
		''' endpoint server.  If you ask for 1 server media services allocates 1 + 1, if you ask for 3 servers 
		''' media services allocates 3 + 1 and so on.
		''' </summary>
		''' <param name="size">the number to add or remove, if negative</param>
		''' <returns>a task</returns>
		Private Function Scale(size As Integer) As Task
			Dim streamEndpoint As IStreamingEndpoint = Me.MediaContext.LiveServices.FindStreamingEndpoint(Me.StreamEndpointName)

			Dim task As Task = Nothing

			If streamEndpoint IsNot Nothing Then
				If streamEndpoint.ScaleUnits.HasValue = False OrElse size < 0 AndAlso (streamEndpoint.ScaleUnits.Value + size) < 0 Then
					Throw New ArgumentOutOfRangeException("you are reducing too many streaming endpoints, value will be negative")
				End If

				task = streamEndpoint.ScaleAsync(streamEndpoint.ScaleUnits.Value + size)
			Else
				Throw New ArgumentException("streaming endpoint not found")
			End If

			Return task
		End Function




        Private Async Sub ResetChannel(Optional cancellationToken As CancellationToken = Nothing)
            Me.Refresh()

            Try
                ' Check if the channel is running, wait if it’s starting or throw if it’s stopped or stopping
                WaitForState(ChannelState.Running)

                ' Reset the channel
                Await Me.Channel.ResetAsync()
            Catch e As OperationCanceledException
                ' Cancelled
                Throw New TimeoutException("Timeout for the reset operation elapsed.", e)
            End Try
            ' any other exception
        End Sub
	End Class
End Namespace
