
' <copyright file="ILocatorExtensions.cs" company="Microsoft">Copyright 2013 Microsoft Corporation</copyright>
' <license>
' Licensed under the Apache License, Version 2.0 (the "License");
' you may not use this file except in compliance with the License.
' You may obtain a copy of the License at
' 
' http://www.apache.org/licenses/LICENSE-2.0
' 
' Unless required by applicable law or agreed to in writing, software
' distributed under the License is distributed on an "AS IS" BASIS,
' WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
' See the License for the specific language governing permissions and
' limitations under the License.
' </license>

Imports System.Globalization
Namespace Microsoft.WindowsAzure.MediaServices.Client

	''' <summary>
	''' Contains extension methods and helpers for the <see cref="ILocator"/> interface.
	''' </summary>
    Public Module ILocatorExtensions
        
        ''' <summary>
        ''' Represents the manifest file extension.
        ''' </summary>
        Public Const ManifestFileExtension As String = ".ism"

        ''' <summary>
        ''' Represents the URL dynamic packaging parameter for HLS.
        ''' </summary>
        Public Const HlsStreamingParameter As String = "(format=m3u8-aapl)"

        ''' <summary>
        ''' Represents the URL dynamic packaging parameter for MPEG-DASH.
        ''' </summary>
        Public Const MpegDashStreamingParameter As String = "(format=mpd-time-csf)"

        Friend Const BaseStreamingUrlTemplate As String = "{0}/{1}/manifest{2}"

        ''' <summary>
        ''' Returns the Smooth Streaming URL of the <paramref name="originLocator"/>; otherwise, null.
        ''' </summary>
        ''' <param name="originLocator">The <see cref="ILocator"/> instance.</param>
        ''' <returns>A <see cref="System.Uri"/> representing the Smooth Streaming URL of the <paramref name="originLocator"/>; otherwise, null.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function GetSmoothStreamingUri(originLocator As ILocator) As Uri
            Return originLocator.GetStreamingUri(String.Empty)
        End Function

        ''' <summary>
        ''' Returns the HLS URL of the <paramref name="originLocator"/>; otherwise, null.
        ''' </summary>
        ''' <param name="originLocator">The <see cref="ILocator"/> instance.</param>
        ''' <returns>A <see cref="System.Uri"/> representing the HLS URL of the <paramref name="originLocator"/>; otherwise, null.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function GetHlsUri(originLocator As ILocator) As Uri
            Return originLocator.GetStreamingUri(HlsStreamingParameter)
        End Function

        ''' <summary>
        ''' Returns the MPEG-DASH URL of the <paramref name="originLocator"/>; otherwise, null.
        ''' </summary>
        ''' <param name="originLocator">The <see cref="ILocator"/> instance.</param>
        ''' <returns>A <see cref="System.Uri"/> representing the MPEG-DASH URL of the <paramref name="originLocator"/>; otherwise, null.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function GetMpegDashUri(originLocator As ILocator) As Uri
            Return originLocator.GetStreamingUri(MpegDashStreamingParameter)
        End Function

        <System.Runtime.CompilerServices.Extension> _
        Private Function GetStreamingUri(originLocator As ILocator, streamingParameter As String) As Uri
            If originLocator Is Nothing Then
                Throw New ArgumentNullException("locator", "The locator cannot be null.")
            End If

            If originLocator.Type <> LocatorType.OnDemandOrigin Then
                Throw New ArgumentException("The locator type must be on-demand origin.", "originLocator")
            End If

            Dim smoothStreamingUri As Uri = Nothing
            Dim asset As IAsset = originLocator.Asset
            Dim manifestAssetFile As IAssetFile = asset.GetManifestAssetFile()
            If manifestAssetFile IsNot Nothing Then
                smoothStreamingUri = New Uri(String.Format(CultureInfo.InvariantCulture, BaseStreamingUrlTemplate, originLocator.Path.TrimEnd("/"c), manifestAssetFile.Name, streamingParameter), UriKind.Absolute)
            End If

            Return smoothStreamingUri
        End Function
    End Module
End Namespace
