
' ----------------------------------------------------------------------
' <copyright file="MainWindow.xaml.cs" company="Microsoft">
' Copyright © Microsoft Corporation.  All rights reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
' FITNESS FOR A PARTICULAR PURPOSE.
' </copyright>
'  <notes>
'                  \\|//
'                  (o o)
' --------------ooO-(_)-Ooo--------------------
'  Authors: Developer Product Evangelist Team (greg.prentice)
'  License: Apache License Version 2.0, January 2004
'                       Oooo
' --------------oooO----(  )-------------------
'               (  )    (_/
'                \_)
'  </notes>
' -----------------------------------------------------------------------

Imports System.Collections.Concurrent
Imports System.Collections.Generic
Imports System.Configuration
Imports System.Diagnostics
Imports System.Linq
Imports System.Reactive.Concurrency
Imports System.Reactive.Disposables
Imports System.Reactive.Linq
Imports System.Text
Imports System.Threading
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Controls
Imports Microsoft.WindowsAzure.MediaServices.Client
Namespace LiveMediaManager

	''' <summary>
	''' Interaction logic for MainWindow.xaml
	''' </summary>
	Public Partial Class MainWindow
		Inherits Window
		Private MediaContext As MediaContext = MediaContext.Create()
		Private cd As CompositeDisposable
		Private currentChannelName As String = Nothing
		Private currentChannel As ChannelContext = Nothing
		Private currentProgramName As String = Nothing
		Private currentProgram As IBroadcastProgram = Nothing
		Private lockMe As New Object()
		Private Subscribe As String
		Private IngestUri As String
		Private PreviewUri As String
		Private LocatorUris As String

		''' <summary>
		''' The list of processes this class manages.
		''' Using a thread safe version of a dictionary. 
		''' </summary>
		Private ReadOnly runningProcesses As IDictionary(Of Task, Action(Of Task)) = New ConcurrentDictionary(Of Task, Action(Of Task))()

		Public Sub New()
			InitializeComponent()

			ReloadTree()

			' Create long running query.
			Me.cd = New CompositeDisposable()

			Dim d As IDisposable
			Dim timed As IObservable(Of Long)

			timed = Observable.Interval(TimeSpan.FromSeconds(2))
			d = timed.ObserveOnDispatcher().Subscribe(AddressOf OnNextUpdateDisplay)
			cd.Add(d)

			timed = Observable.Interval(TimeSpan.FromSeconds(3))
			d = timed.ObserveOn(Scheduler.[Default]).Subscribe(AddressOf OnNextGetDisplayValues)
			cd.Add(d)
		End Sub

		''' <summary>
		''' MSDN Article Code: describes the straight forward method of setting up a live streaming
		''' channel.
		''' </summary>
		Private Sub CreateContosoLiveEvent()
			Dim liveScope As String = ConfigurationManager.AppSettings("MediaLiveScope")
			' Note: Default to production endpoints for final code.
			Dim liveRestEndpoint As New Uri(ConfigurationManager.AppSettings("MediaLiveUri"))
			Dim liveAcsEndpoint As New Uri(ConfigurationManager.AppSettings("MediaLiveAcs"))

			' ====================================================================
			' Task 1 connect to service
			' --------------------------------------------------------------------   
			Dim liveAccount As String = ConfigurationManager.AppSettings("MediaLiveAccount")
			Dim liveKey As String = ConfigurationManager.AppSettings("MediaLiveKey")

			' Account name
			Dim LiveServices As New CloudMediaContext(liveAccount, liveKey)
			' Account key
			' ------------------------------- end --------------------------------
			Dim iplist As New List(Of IPRange)()
			Dim range As New IPRange()

			' ====================================================================
			' Task 2 create streaming end point if it doesn't exist
			' --------------------------------------------------------------------     
			Dim streamingName As String = "abcdefg"
			Dim streaming As IStreamingEndpoint = LiveServices.FindStreamingEndpoint(streamingName)

			If streaming Is Nothing Then
                Dim seOptions As New StreamingEndpointCreationOptions() With { _
                     .Name = streamingName, _
                     .Description = "xyz", _
                     .ScaleUnits = 2, _
                     .AccessControl = New StreamingEndpointAccessControl(), _
                     .CacheControl = New StreamingEndpointCacheControl() _
                }

				' Streaming security ip restriction.
				range.Address = System.Net.IPAddress.Parse("0.0.0.0")
				range.SubnetPrefixLength = 0
				iplist.Add(range)
				seOptions.AccessControl.IPAllowList = iplist
				seOptions.CacheControl.MaxAge = New TimeSpan(0, 15, 0)

				Dim streamingTask As Task(Of IStreamingEndpoint) = LiveServices.StreamingEndpoints.CreateAsync(streamingName, 2)
				streamingTask.Wait()
				streaming = streamingTask.Result
			End If
			If streaming.State = StreamingEndpointState.Stopped Then
				streaming.StartAsync().Wait()
			End If
			' ------------------------------- end --------------------------------

			' ====================================================================
			' Task 3 Create the channel
			' --------------------------------------------------------------------
			Dim channelName As String = "ContosoBluesChannel"
			Dim channel As IChannel = LiveServices.FindChannel(channelName)
			If channel IsNot Nothing Then
				Debug.WriteLine("Channel already exists!")
				Return
			End If

			' Look into this method as it cosolidate changes between private and the public preview
			'  for the live streaming api.
			Dim options As ChannelCreationOptions = GetDefaultChannelOptions(channelName)

			' Create the channel.
			Dim taskChannel As Task(Of IChannel) = LiveServices.Channels.CreateAsync(options)

			taskChannel.Wait()

			channel = taskChannel.Result
			' ------------------------------- end --------------------------------

			' ====================================================================
			' Task 4 Create the programs with locators
			' --------------------------------------------------------------------
			' Define the program name, DVR window and estimated duration in minutes.
			'  NOTE: DVR window value mostlikely will be removed when service reaches public preview.
			Dim programSettings As Tuple(Of String, Integer)() = New Tuple(Of String, Integer)() {New Tuple(Of String, Integer)("Concert1", 480), New Tuple(Of String, Integer)("Vip1", 75), New Tuple(Of String, Integer)("Act1", 90), New Tuple(Of String, Integer)("Act2", 165), New Tuple(Of String, Integer)("Vip2", 120)}

			For Each programSetting As Tuple(Of String, Integer) In programSettings
				Dim asset As IAsset = Nothing

				Try
					' Create the asset that is used to persist the video streamed 
					' while the program is running and VOD.
					asset = LiveServices.Assets.Create(programSetting.Item1 + "_asset", AssetCreationOptions.None)


					Dim programOptions As New ProgramCreationOptions()

					programOptions.Name = programSetting.Item1
					programOptions.Description = "program description"
					' Default name is a guid else you can specify more friendly name, must be unique
					programOptions.ManifestName = programSetting.Item1.ToLower() + "manifest"
					programOptions.AssetId = asset.Id
					programOptions.ArchiveWindowLength = TimeSpan.FromMinutes(programSetting.Item2)

					Dim taskProgram As Task(Of IProgram) = channel.Programs.CreateAsync(programOptions)

					taskProgram.Wait()

					LiveServices.CreateLocators(asset, TimeSpan.FromDays(30))
				Catch exp As Exception
					Debug.WriteLine(exp.Message)
				End Try
			Next
			' ------------------------------- end --------------------------------
		End Sub


		Private Function GetDefaultChannelOptions(channelName As String) As ChannelCreationOptions
			If String.IsNullOrEmpty(channelName) Then
				Throw New ArgumentNullException("channelName")
			End If

			Dim iplist As New List(Of IPRange)()
			Dim range As New IPRange()
			range.Address = System.Net.IPAddress.Parse("0.0.0.0")
			range.Name = "AllowsAllChangeMe"
			range.SubnetPrefixLength = 0
			iplist.Add(range)

			' NOTE: Changed from original Article
            Dim options As New ChannelCreationOptions() With { _
                 .Name = channelName, _
                 .Preview = New ChannelPreview(), _
                 .Input = New ChannelInput(), _
                 .Output = New ChannelOutput() _
            }

			options.Input.StreamingProtocol = StreamingProtocol.RTMP
			options.Input.AccessControl = New ChannelAccessControl()
			options.Input.AccessControl.IPAllowList = iplist
			options.Preview.AccessControl = New ChannelAccessControl()
			options.Preview.AccessControl.IPAllowList = iplist

			' Needed to dynamux to HLS
			' Do you know your smooth or fragmented mp4 duration then you would set
			'  Hls.FragmentsPerSegment (FPS) equal ratio of FPS:HLS:
			'      1 = one fragment to one HLS segment
			'      3 = three fragments to one HLS segment
			'      5 = five fragments to one HLS segment ...
			'  If you are doing ad insertion you need to be accurate. So always set this value to 1;
			'  ad is always inserted at the fragment level. A marker is inserted and then received
			'  by the client.
			options.Output.Hls = New ChannelOutputHls()
			options.Output.Hls.FragmentsPerSegment = 3

			Return options
		End Function

		Private Sub ReloadTree()
			While Me.ChannelTree.Items.Count > 0
				Me.ChannelTree.Items.RemoveAt(0)
			End While

            'Dim channels = Me.MediaContext.LiveServices.Channels.ToObservable(Of IChannel)()
            'channels.ObserveOnDispatcher().Subscribe(AddressOf OnNext, AddressOf OnError, AddressOf OnCompleted)

		End Sub

		''' <summary>
		''' Thread that updates our UI variables.
		''' </summary>
		''' <param name="i">current iteration</param>
		Private Sub OnNextGetDisplayValues(i As Long)
			' Clean up finished threads
			For Each kvp As KeyValuePair(Of Task, Action(Of Task)) In Me.runningProcesses
				Try
					If kvp.Key.Wait(0) Then
						' maybe tracking items that don't have an action.
						If kvp.Value IsNot Nothing Then
							' Most of my actions have to do with updating the UI.
							'  Just make sure I'm on that thread when invoking.
							Me.Dispatcher.Invoke(kvp.Value, kvp.Key)
						End If
						Me.runningProcesses.Remove(kvp.Key)
					End If
				Catch generatedExceptionName As Exception
					' If we are here there was a bad error in a task.
					'  For now just remove it, but you may want to keep a break
					'  point set here for trouble shooting.
					Me.runningProcesses.Remove(kvp.Key)
				End Try
			Next

			If String.IsNullOrEmpty(Me.currentChannelName) Then
				Return
			End If

			Try
				Dim channel As ChannelContext

				SyncLock lockMe

					Me.currentChannel = MediaContext.FindChannel(Me.currentChannelName)
					channel = Me.currentChannel
				End SyncLock

				If channel IsNot Nothing Then

					Me.IngestUri = String.Empty
					Me.PreviewUri = String.Empty

					Dim ep As ChannelEndpoint = channel.Channel.Preview.Endpoints.FirstOrDefault()
					Me.PreviewUri = ep.Url.ToString()

					ep = channel.Channel.Input.Endpoints.FirstOrDefault()

					Me.IngestUri = ep.Url.ToString()

					Dim sb As New StringBuilder()
					sb.AppendFormat("Channel = Name: {0}, State: {1} ", channel.Channel.Name, channel.Channel.State)

					Dim program As IBroadcastProgram = Nothing

					SyncLock lockMe
						Me.currentProgram = Me.FindProgram(channel, Me.currentProgramName)
						program = Me.currentProgram
					End SyncLock

					Me.LocatorUris = String.Empty

					If program IsNot Nothing Then
						sb.AppendFormat(" + Program = Name: {0}, State: {1}", program.Program.Name, program.Program.State)
						Dim asset As IAsset = channel.MediaContext.LiveServices.FindAsset(program.Program)

						' ----------------------------------------------------------------------------------
						' Used to fixup stream endpoint names
						' TODO: remove once this is fixed in SDK APIs
						' ----------------------------------------------------------------------------------
						Dim streaming As IStreamingEndpoint = Me.MediaContext.LiveServices.FindStreamingEndpoint(channel.StreamEndpointName)
						Dim streamLocator As ILocator = asset.Locators.FirstOrDefault()
						If streaming IsNot Nothing AndAlso streamLocator IsNot Nothing Then
							' ----------------------------------------------------------------------------------

							Dim sbl As New StringBuilder()

							For Each kvp As KeyValuePair(Of VideoFormat, String) In channel.MediaContext.LiveServices.CreateStreamingURIs(asset)
								' ----------------------------------------------------------------------------------
								' Used to fixup stream endpoint names
								' TODO: remove once this is fixed in SDK APIs
								' ----------------------------------------------------------------------------------
								Dim newUrl As String = kvp.Value.Replace(streamLocator.BaseUri, "http://" + streaming.HostName)
								' ----------------------------------------------------------------------------------
								sbl.AppendFormat("{0} " & vbLf, newUrl)
							Next
							Me.LocatorUris = sbl.ToString()
						End If
					End If

					Me.Subscribe = sb.ToString()
				Else
					Me.Subscribe = String.Empty
					Me.IngestUri = String.Empty
					Me.PreviewUri = String.Empty
					Me.LocatorUris = String.Empty
				End If
			Catch ex As Exception
				System.Diagnostics.Debug.WriteLine(ex.Message)
			End Try
		End Sub

		''' <summary>
		''' This call executes on the UI thread and thus allows us to update the UI state.
		''' We also use this as an opportune time to check, execute and remove task that may
		''' be executing.
		''' </summary>
		''' <param name="i">current iteration</param>
		Private Sub OnNextUpdateDisplay(i As Long)
			SyncLock lockMe
				Dim item As TreeViewItem = FindCurrentChannelItem()
				If item IsNot Nothing Then
					Me.currentChannelName = item.Header.ToString()
				End If
				item = FindCurrentProgramItem()
				If item IsNot Nothing Then
					Me.currentProgramName = item.Header.ToString()
				End If
			End SyncLock

			Me.textSubscribe.Text = Me.Subscribe
			Me.textIngestUri.Text = Me.IngestUri
			Me.textPreviewUri.Text = Me.PreviewUri
			Me.textLocatorUris.Text = Me.LocatorUris

			If Me.runningProcesses.Count > 0 Then
				' Don't enable buttons if we still have task pending.
				Return
			End If

			EnableButtons()
		End Sub


		''' <summary>
		''' Used to fill our tree view.
		''' </summary>
		''' <param name="channel">the channel information</param>
		Private Sub OnNext(channel As IChannel)

			Dim item As New TreeViewItem()
			item.Header = channel.Name
			Dim index As Integer = Me.ChannelTree.Items.Add(item)
			For Each program As IProgram In channel.Programs
				Dim item1 As New TreeViewItem()
				item1.Header = program.Name
				item.Items.Add(item1)
			Next
		End Sub

		''' <summary>
		''' Used to signal we are done filling our tree view.
		''' </summary>
		Private Sub OnCompleted()
			If Me.ChannelTree.Items.Count > 0 Then
				Dim item As TreeViewItem = TryCast(Me.ChannelTree.Items(0), TreeViewItem)
				item.IsSelected = True
			End If
			Me.textSubscribe.Text = "Finished Getting Channels!"
		End Sub

		''' <summary>
		''' Used if an error occurs while filling tree view.
		''' </summary>
		''' <param name="ex">The exception thrown</param>
		Private Sub OnError(ex As Exception)
			Me.textSubscribe.Text = "ERROR Getting channels!" + ex.Message
		End Sub


		''' <summary>
		''' Helper function to get the channel based on the selection in the tree view.
		''' </summary>
		''' <returns>the channel context or null if not found</returns>
		Private Function FindSelectedChannelContext() As ChannelContext
			Dim item As TreeViewItem = FindCurrentChannelItem()
			If item IsNot Nothing Then
				Dim name As String = item.Header.ToString()

				If Not String.IsNullOrEmpty(name) Then
					Return MediaContext.FindChannel(name)
				End If
			End If
			Return Nothing
		End Function

		Private Function FindCurrentChannelItem() As TreeViewItem
			If Me.ChannelTree.SelectedItem Is Nothing Then
				Return Nothing
			End If

			Dim item As TreeViewItem = TryCast(Me.ChannelTree.SelectedItem, TreeViewItem)

			' Is this a program?
			If item IsNot Nothing AndAlso item.Parent IsNot Nothing Then
				If GetType(TreeViewItem) = item.Parent.[GetType]() Then
					' We have a program so get channel
					item = TryCast(item.Parent, TreeViewItem)
				End If
			End If

			Return item
		End Function

		Private Function FindSelectedProgram(channel As ChannelContext) As IBroadcastProgram
			Dim item As TreeViewItem = FindCurrentProgramItem()
			If item Is Nothing Then
				Return Nothing
			End If
			Return FindProgram(channel, item.Header.ToString())
		End Function

		Private Function FindProgram(channel As ChannelContext, name As String) As IBroadcastProgram
			If channel Is Nothing Then
				Return Nothing
			End If
			If String.IsNullOrEmpty(name) Then
				Return Nothing
			End If
			Return channel.BroadcastSchedule.Find(name)
		End Function

		''' <summary>
		''' Helper function to get the channel based on the selection in the tree view.
		''' </summary>
		''' <returns>the channel context or null if not found</returns>
		Private Function FindCurrentProgramItem() As TreeViewItem
			If Me.ChannelTree.SelectedItem Is Nothing Then
				Return Nothing
			End If
			Dim item As TreeViewItem = TryCast(Me.ChannelTree.SelectedItem, TreeViewItem)


			' We are currently at the program.
			If item.Parent IsNot Nothing Then
				If GetType(TreeViewItem) = item.Parent.[GetType]() Then
					Return item
				End If
			End If

			Return Nothing
		End Function

		Private Sub btnCreateChannel_Click(sender As Object, e As RoutedEventArgs)

			Try
				' We are creating so pass null.
				Dim window As New ChannelEditor(Nothing)

				Dim ok As System.Nullable(Of Boolean) = window.ShowDialog()

				If ok.HasValue AndAlso ok.Value Then
					Dim uniqueName As Guid = Guid.NewGuid()
					Dim channelName As String = window.textName.Text
					Dim description As String = window.textDescription.Text

					If MediaContext.IsChannelDefined(channelName) = False Then
						DisableAllButtons()

						Dim options As ChannelCreationOptions = GetDefaultChannelOptions(channelName)
						options.Description = description

						Dim task As Task(Of ChannelContext) = MediaContext.CreateChannel(options)

						Dim action As Action(Of Task) = Function(t) 
						If task.Result IsNot Nothing Then
							Dim item As New TreeViewItem()
							item.Header = channelName
							Dim index As Integer = Me.ChannelTree.Items.Add(item)
							item.IsSelected = True
							EnableButtons()
						End If

End Function

						Me.runningProcesses.Add(task, action)
					Else
						MessageBox.Show("Channel Already Exists!", "Channel Error")
					End If
				End If
			Catch ex As Exception
				' Here if nothing selected or something else went wrong.
				MessageBox.Show(ex.Message)
			End Try
		End Sub

		Private Sub btnDeleteChannel_Click(sender As Object, e As RoutedEventArgs)
			Try
				If MessageBox.Show("Do you want to delete the channel?", "Warning Deleting Channel", MessageBoxButton.OKCancel) = MessageBoxResult.OK Then

					Dim channel As ChannelContext = Me.FindSelectedChannelContext()
					If channel IsNot Nothing Then
						If channel.Channel.Programs.Count() > 0 Then
							MessageBox.Show("You must stop and delete all of the programs before deleting a channel.", "Warning programs exist")
							Return
						End If

						DisableAllButtons()

						Dim deleteAction As Action = Function() 
						channel.[Stop]().Wait()
						channel.Delete().Wait()

End Function

						Dim action As Action(Of Task) = Function(t) 
						Me.ChannelTree.Items.Remove(Me.ChannelTree.SelectedItem)
						EnableButtons()

End Function

						Dim task As New Task(deleteAction)
						task.Start()

						Me.runningProcesses.Add(task, action)
					End If
				End If
			Catch ex As Exception
				' Here if nothing selected or something else went wrong.
				MessageBox.Show(ex.Message)
			End Try
		End Sub

		Private Sub btnStopChannel_Click(sender As Object, e As RoutedEventArgs)
			Try

				If MessageBox.Show("Do you want to Stop the channel?", "Stop Channel", MessageBoxButton.OKCancel) = MessageBoxResult.OK Then
					Dim channel As ChannelContext = Me.FindSelectedChannelContext()
					If channel IsNot Nothing Then
						DisableAllButtons()

						Dim task As Task = channel.[Stop]()

						Dim action As Action(Of Task) = Function(t) 
						EnableButtons()

End Function

						Me.runningProcesses.Add(task, action)
					End If
				End If
			Catch ex As Exception
				' Here if nothing selected or something else went wrong.
				MessageBox.Show(ex.Message)
			End Try
		End Sub

		Private Sub btnStartChannel_Click(sender As Object, e As RoutedEventArgs)
			Try
				If MessageBox.Show("Do you want to Start the channel?", "Start Channel", MessageBoxButton.OKCancel) = MessageBoxResult.OK Then
					Dim channel As ChannelContext = Me.FindSelectedChannelContext()
					If channel IsNot Nothing Then
						DisableAllButtons()
						Dim task As Task = channel.Start()
						Me.runningProcesses.Add(task, Function(t) 
						EnableButtons()

End Function)
					End If
				End If
			Catch ex As Exception
				' Here if nothing selected or something else went wrong.
				MessageBox.Show(ex.Message)
			End Try
		End Sub

		Private Sub btnCreateStreamingEndpoint_Click(sender As Object, e As RoutedEventArgs)
			Try
				Dim channel As ChannelContext = Me.FindSelectedChannelContext()
				If channel IsNot Nothing Then
					DisableAllButtons()
					Dim task As Task = channel.CreateStreamingEndpointAndStart()
					Me.runningProcesses.Add(task, Function(t) 
					EnableButtons()

End Function)
				End If
			Catch ex As Exception
				' Here if nothing selected or something else went wrong.
				MessageBox.Show(ex.Message)
			End Try
		End Sub

		Private Sub btnDeleteStreamingEndpoint_Click(sender As Object, e As RoutedEventArgs)
			Try
				Dim channel As ChannelContext = Me.FindSelectedChannelContext()
				If channel IsNot Nothing Then
					DisableAllButtons()
					Dim task As Task = channel.StopAndDeleteStreamingEndpoints()
					Me.runningProcesses.Add(task, Function(t) 
					EnableButtons()

End Function)
				End If
			Catch ex As Exception
				' Here if nothing selected or something else went wrong.
				MessageBox.Show(ex.Message)
			End Try
		End Sub

		Private Sub btnCreateProgram_Click(sender As Object, e As RoutedEventArgs)
			' We are creating so pass null.
			Dim window As New ProgramEditor()

			Dim ok As System.Nullable(Of Boolean) = window.ShowDialog()

			If ok.HasValue AndAlso ok.Value Then
				Dim channel As ChannelContext = Me.FindSelectedChannelContext()
				If channel IsNot Nothing Then

					Dim name As String = window.txtName.Text

					Dim program As IBroadcastProgram

					program = channel.BroadcastSchedule.Find(name)

					If program IsNot Nothing Then
						MessageBox.Show("A program already exists with this name!")
						Return
					End If

					DisableAllButtons()

					Dim description As String = window.txtDescription.Text

					Dim duration As Integer = Int16.Parse(window.txtDuration.Text)
					Dim dvr As Integer = Int16.Parse(window.txtDvrWindow.Text)
					Dim enabled As Boolean = window.chkEnableArchive.IsChecked.Value

					Dim func As Func(Of IBroadcastProgram) = Function() 
					Dim settings As New ProgramCreationOptions()
					settings.Name = name
					settings.Description = description
					settings.ArchiveWindowLength = TimeSpan.FromMinutes(dvr)
					'settings.EstimatedDuration = TimeSpan.FromMinutes(duration);
					Dim t As Task(Of IBroadcastProgram) = channel.BroadcastSchedule.Create(settings)
					t.Wait()
					Return program

End Function

					Dim action As Action(Of Task) = Function(t) 
					' Add the program to ui
					Dim item As TreeViewItem = FindCurrentChannelItem()
					Dim item1 As New TreeViewItem()
					item1.Header = name
					item.Items.Add(item1)
					EnableButtons()

End Function

					Dim task As New Task(Of IBroadcastProgram)(func)
					task.Start()

					Me.runningProcesses.Add(task, action)
				End If
			End If
		End Sub

		Private Sub btnStartProgram_Click(sender As Object, e As RoutedEventArgs)
			Try
				If MessageBox.Show("Do you want to start the Program?", "Start Program", MessageBoxButton.OKCancel) = MessageBoxResult.OK Then
					Dim channel As ChannelContext = Me.FindSelectedChannelContext()
					If channel IsNot Nothing Then
						Dim program As IBroadcastProgram = FindSelectedProgram(channel)

						If program IsNot Nothing Then
							DisableAllButtons()
							Dim action As Action(Of Task) = Function(t) 
							EnableButtons()

End Function

							Me.runningProcesses.Add(program.Start(), action)
						End If
					End If
				End If
			Catch ex As Exception
				' Here if nothing selected or something else went wrong.
				MessageBox.Show(ex.Message)
			End Try
		End Sub

		Private Sub btnStopProgram_Click(sender As Object, e As RoutedEventArgs)
			Try
				If MessageBox.Show("Do you want to Stop the program?", "Stop Program", MessageBoxButton.OKCancel) <> MessageBoxResult.OK Then
					Return
				End If
				Dim channel As ChannelContext = Me.FindSelectedChannelContext()
				If channel IsNot Nothing Then
					Dim program As IBroadcastProgram = FindSelectedProgram(channel)

					If program IsNot Nothing Then
						DisableAllButtons()
						Dim action As Action(Of Task) = Function(t) 
						EnableButtons()

End Function

						Me.runningProcesses.Add(program.[Stop](), action)
					End If
				End If
			Catch ex As Exception
				' Here if nothing selected or something else went wrong.
				MessageBox.Show(ex.Message)
			End Try
		End Sub

		Private Sub btnDeleteProgram_Click(sender As Object, e As RoutedEventArgs)
			Try
				If MessageBox.Show("Do you want to delete the program?", "Delete Program", MessageBoxButton.OKCancel) <> MessageBoxResult.OK Then
					Return
				End If

				Dim deleteAsset As Boolean = MessageBox.Show("Do you also want to delete the video asset?", "Delete Program", MessageBoxButton.YesNo) = MessageBoxResult.Yes

				Dim channel As ChannelContext = Me.FindSelectedChannelContext()
				If channel IsNot Nothing Then
					Dim program As IBroadcastProgram = FindSelectedProgram(channel)

					If program IsNot Nothing Then
						DisableAllButtons()

						Dim func As Action = Function() 
						program.Delete(deleteAsset)

End Function

						Dim action As Action(Of Task) = Function(t) 
						Dim item As TreeViewItem = FindCurrentChannelItem()
						item.Items.Remove(Me.ChannelTree.SelectedItem)
						EnableButtons()

End Function

						Dim task As New Task(func)
						task.Start()
						Me.runningProcesses.Add(task, action)
					End If
				End If
			Catch ex As Exception
				' Here if nothing selected or something else went wrong.
				MessageBox.Show(ex.Message)
			End Try

		End Sub

		Private Sub btnCreateLocator_Click(sender As Object, e As RoutedEventArgs)
			Dim channel As ChannelContext = Me.FindSelectedChannelContext()

			If channel IsNot Nothing Then
				Dim program As IBroadcastProgram = FindSelectedProgram(channel)

				If program IsNot Nothing Then
					DisableAllButtons()

					Dim action As Action(Of Task) = Function(t) 
					EnableButtons()

End Function
					Me.runningProcesses.Add(channel.CreateLocators(program.Program), action)
				End If
			End If
		End Sub

		Private Sub btnDeleteLocator_Click(sender As Object, e As RoutedEventArgs)
			Dim channel As ChannelContext = Me.FindSelectedChannelContext()

			If channel IsNot Nothing Then
				Dim program As IBroadcastProgram = FindSelectedProgram(channel)

				If program IsNot Nothing Then
					DisableAllButtons()

					Dim action As Action(Of Task) = Function(t) 
					EnableButtons()

End Function
					Me.runningProcesses.Add(channel.DeleteLocators(program.Program), action)
				End If
			End If
		End Sub

		Private Sub ChannelTree_SelectedItemChanged(sender As Object, e As RoutedPropertyChangedEventArgs(Of Object))
			DisableAllButtons()

			SyncLock lockMe
				Me.Subscribe = String.Empty
				Me.IngestUri = String.Empty
				Me.PreviewUri = String.Empty
				Me.LocatorUris = String.Empty
				Me.textSubscribe.Text = String.Empty
				Me.textIngestUri.Text = String.Empty
				Me.textPreviewUri.Text = String.Empty
				Me.textLocatorUris.Text = String.Empty
				Me.currentChannelName = Nothing
				Me.currentProgramName = Nothing
			End SyncLock
		End Sub


		Private Sub DisableAllButtons()
			Me.ChannelTree.IsEnabled = False
			Me.btnBigRed.IsEnabled = False
			Me.btnStopChannel.IsEnabled = False
			Me.btnDeleteChannel.IsEnabled = False
			Me.btnStartChannel.IsEnabled = False
			Me.btnStopChannel.IsEnabled = False
			Me.btnCreateProgram.IsEnabled = False
			Me.btnDeleteProgram.IsEnabled = False
			Me.btnStopProgram.IsEnabled = False
			Me.btnStartProgram.IsEnabled = False
			Me.btnCreateOrigin.IsEnabled = False
			Me.btnDeleteOrigin.IsEnabled = False
			Me.btnCreateLocator.IsEnabled = False
			Me.btnDeleteLocator.IsEnabled = False

		End Sub

		Private Sub EnableButtons()
			If Me.currentProgram IsNot Nothing Then
				SetProgramSelected(Me.currentProgram.Program.State)
			ElseIf Me.currentChannel IsNot Nothing Then
				SetChannelSelected(Me.currentChannel.Channel.State)
			End If
			Me.ChannelTree.IsEnabled = True
			Me.btnBigRed.IsEnabled = True
		End Sub

		Private Sub SetChannelSelected(state As ChannelState)
			DisableAllButtons()

			Select Case state
				Case ChannelState.Running
					Me.btnStopChannel.IsEnabled = True
					Me.btnCreateProgram.IsEnabled = True
					Exit Select
				Case ChannelState.Stopped
					Me.btnDeleteChannel.IsEnabled = True
					Me.btnStartChannel.IsEnabled = True
					Me.btnCreateProgram.IsEnabled = True
					Exit Select
			End Select

			Me.btnCreateOrigin.IsEnabled = True
			Me.btnDeleteOrigin.IsEnabled = True
		End Sub


		Private Sub SetProgramSelected(state As ProgramState)
			DisableAllButtons()

			Select Case state
				Case ProgramState.Running
					Me.btnStopProgram.IsEnabled = True
					Exit Select
				Case ProgramState.Stopped
					Me.btnDeleteProgram.IsEnabled = True
					Me.btnStartProgram.IsEnabled = True
					Exit Select
			End Select

			Me.btnCreateOrigin.IsEnabled = True
			Me.btnDeleteOrigin.IsEnabled = True
			Me.btnCreateLocator.IsEnabled = True
			Me.btnDeleteLocator.IsEnabled = True
		End Sub

		Private Sub Window_Closing(sender As Object, e As System.ComponentModel.CancelEventArgs)
			Me.cd.Dispose()
		End Sub

		Private Sub btnBigRed_Click(sender As Object, e As RoutedEventArgs)
			DisableAllButtons()

			Dim action As Action(Of Task) = Function(t) 
			ReloadTree()
			EnableButtons()

End Function

			Dim task As New Task(AddressOf CreateContosoLiveEvent)
			task.Start()
			Me.runningProcesses.Add(task, action)

		End Sub
	End Class
End Namespace
