
' ----------------------------------------------------------------------
' <copyright file="MediaContext.cs" company="Microsoft">
' Copyright © Microsoft Corporation.  All rights reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
' FITNESS FOR A PARTICULAR PURPOSE.
' </copyright>
'  <notes>
'                  \\|//
'                  (o o)
' --------------ooO-(_)-Ooo--------------------
'  Authors: Developer Product Evangelist Team (greg.prentice)
'  License: Apache License Version 2.0, January 2004
'                       Oooo
' --------------oooO----(  )-------------------
'               (  )    (_/
'                \_)
'  </notes>
' -----------------------------------------------------------------------

Imports System.Collections.Generic
Imports System.Configuration
Imports System.Linq
Imports System.Security.Cryptography
Imports System.Text
Imports System.Threading
Imports System.Threading.Tasks
Imports Microsoft.WindowsAzure.MediaServices.Client
Namespace LiveMediaManager

	''' <summary>
	''' Defines a wrapper class for the account credentials and connection to the Azure Media Services.
	''' </summary>
	Public Class MediaContext
		Implements IDisposable
		''' <summary>
		''' Initializes a new instance of the <see cref="MediaContext"/> class.
		''' </summary>
		Public Sub New()
			Me.Retries = 100
			Me.SleepMilliseconds = 1000
			Me.Initialize(ConfigurationManager.AppSettings("MediaLiveAccount"), ConfigurationManager.AppSettings("MediaLiveKey"), ConfigurationManager.AppSettings("MediaLiveScope"), New Uri(ConfigurationManager.AppSettings("MediaLiveUri")), New Uri(ConfigurationManager.AppSettings("MediaLiveAcs")))
		End Sub

		''' <summary>
		''' Initializes a new instance of the <see cref="MediaContext"/> class.
		''' </summary>
		''' <param name="liveAccount">the account</param>
		''' <param name="liveKey">they key</param>
		''' <param name="liveScope">the scope</param>
		''' <param name="liveRestEndpoint">the rest end point</param>
		''' <param name="liveAcsEndpoint">the ACS end point</param>
		Public Sub New(liveAccount As String, liveKey As String, liveScope As String, liveRestEndpoint As Uri, liveAcsEndpoint As Uri)
			Me.Retries = 100
			Me.SleepMilliseconds = 1000
			Me.Initialize(liveAccount, liveKey, liveScope, liveRestEndpoint, liveAcsEndpoint)
		End Sub

		''' <summary>
		''' Hard coded streaming endpoint name. Should be changed
		''' and configurable in live production.
		''' </summary>
		Public Const StreamingEndpointName As String = "abcdefg"

		''' <summary>
		''' Gets the Azure media services context object.
		''' </summary>
		Public Property LiveServices() As CloudMediaContext
			Get
				Return m_LiveServices
			End Get
			Private Set
				m_LiveServices = Value
			End Set
		End Property
		Private m_LiveServices As CloudMediaContext

		''' <summary>
		''' Gets the live media account name.
		''' </summary>
		Public Property LiveAccount() As String
			Get
				Return m_LiveAccount
			End Get
			Private Set
				m_LiveAccount = Value
			End Set
		End Property
		Private m_LiveAccount As String

		''' <summary>
		''' Gets the live media account key.
		''' </summary>
		Public Property LiveKey() As String
			Get
				Return m_LiveKey
			End Get
			Private Set
				m_LiveKey = Value
			End Set
		End Property
		Private m_LiveKey As String

		''' <summary>
		''' Gets a value to the live reset endpoint.
		''' TODO: this goes away for preview.
		''' </summary>
		Public Property LiveRestEndpoint() As Uri
			Get
				Return m_LiveRestEndpoint
			End Get
			Private Set
				m_LiveRestEndpoint = Value
			End Set
		End Property
		Private m_LiveRestEndpoint As Uri

		''' <summary>
		''' Gets a value to the live credential endpoint.
		''' TODO: this goes away for preview.
		''' </summary>
		Public Property LiveAcsEndpoint() As Uri
			Get
				Return m_LiveAcsEndpoint
			End Get
			Private Set
				m_LiveAcsEndpoint = Value
			End Set
		End Property
		Private m_LiveAcsEndpoint As Uri

		''' <summary>
		''' Gets the scope of the account. 
		''' TODO: this goes away for preview
		''' </summary>
		Public Property LiveScope() As String
			Get
				Return m_LiveScope
			End Get
			Private Set
				m_LiveScope = Value
			End Set
		End Property
		Private m_LiveScope As String

		''' <summary>
		''' Gets or sets the number of retries when waiting for a state change.
		''' </summary>
		Public Property Retries() As Integer
			Get
				Return m_Retries
			End Get
			Set
				m_Retries = Value
			End Set
		End Property
		Private m_Retries As Integer

		''' <summary>
		''' Gets or sets the number of milliseconds to wait when testing a state change.
		''' </summary>
		Public Property SleepMilliseconds() As Integer
			Get
				Return m_SleepMilliseconds
			End Get
			Set
				m_SleepMilliseconds = Value
			End Set
		End Property
		Private m_SleepMilliseconds As Integer

		''' <summary>
		''' Creates a media context.
		''' </summary>
		''' <returns>a media context</returns>
		Public Shared Function Create() As MediaContext
			Return New MediaContext()
		End Function

		''' <summary>
		''' Locates a channel.
		''' </summary>
		''' <param name="name">the name of the channel</param>
		''' <returns>A channel context</returns>
		Public Shared Function FindChannel(name As String) As ChannelContext
			If String.IsNullOrEmpty(name) Then
				Throw New ArgumentNullException("name")
			End If

			Dim mediaContext__1 As New MediaContext()

            Dim channel As IChannel = (From x In mediaContext__1.LiveServices.Channels Where x.Name = name).FirstOrDefault()

			Dim channelContext As ChannelContext = Nothing

			If channel IsNot Nothing Then
				channelContext = New ChannelContext(channel, MediaContext.StreamingEndpointName, mediaContext__1)
			End If

			Return channelContext
		End Function

		''' <summary>
		''' Checks to see if a channel exist with a given name.
		''' </summary>
		''' <param name="name">name of the channel</param>
		''' <returns>true if this channel is defined</returns>
		Public Shared Function IsChannelDefined(name As String) As Boolean
			If String.IsNullOrEmpty(name) Then
				Throw New ArgumentNullException("name")
			End If

			Dim channel As IChannel = Nothing

			Using mediaContext As New MediaContext()
                channel = (From x In mediaContext.LiveServices.Channels Where x.Name = name).FirstOrDefault()
			End Using

			Return channel IsNot Nothing
		End Function

		''' <summary>
		''' Attempts to find a channel and if it does not exist, then creates one.
		''' </summary>
		''' <param name="name">name of the channel</param>
		''' <param name="description">description of channel</param>
		''' <param name="size">size of channel</param>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Public Shared Function FindOrCreateChannel(name As String, options As ChannelCreationOptions) As Task(Of ChannelContext)
			Dim task As Task(Of ChannelContext)

			Dim func As Func(Of ChannelContext) = Function() 
			Return FindChannel(name)

End Function

			task = New Task(Of ChannelContext)(func)
			task.Start()
			task.Wait()

			If task.Result Is Nothing Then
				task = MediaContext.CreateChannel(options)
			End If

			Return task
		End Function

		''' <summary>
		''' Creates a channel, which is the first requirement of setting up a streaming scenario.
		''' </summary>
		''' <param name="name">the name of the channel</param>
		''' <param name="description">a description of the channel</param>
		''' <param name="size">type of channel to create</param>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Public Shared Function CreateChannel(options As ChannelCreationOptions) As Task(Of ChannelContext)
			If options Is Nothing Then
				Throw New ArgumentNullException("options")
			End If

			If String.IsNullOrEmpty(options.Name) Then
				Throw New ArgumentNullException("options.Name")
			End If

			Dim func As Func(Of ChannelContext) = Function() 
			Dim mediaContext__1 As New MediaContext()

			' Create the channel.
			Dim channel As Task(Of IChannel) = mediaContext__1.LiveServices.Channels.CreateAsync(options)

			' Wait for the operation to complete
			channel.Wait()

			Dim channelContext As ChannelContext = Nothing

			If channel.Result IsNot Nothing Then
				channelContext = New ChannelContext(channel.Result, MediaContext.StreamingEndpointName, mediaContext__1)
			End If

			Return channelContext

End Function

			Dim tsk As New Task(Of ChannelContext)(func)
			tsk.Start()
			Return tsk
		End Function

		''' <summary>
		''' Initializes the LiveServices to a new set of credentials.
		''' </summary>
		''' <param name="liveAccount">the account</param>
		''' <param name="liveKey">they key</param>
		''' <param name="liveScope">the scope</param>
		''' <param name="liveRestEndpoint">the rest end point</param>
		''' <param name="liveAcsEndpoint">the ACS end point</param>
		Public Sub Initialize(liveAccount As String, liveKey As String, liveScope As String, liveRestEndpoint As Uri, liveAcsEndpoint As Uri)
			If String.IsNullOrWhiteSpace(liveAccount) Then
				Throw New ArgumentNullException("liveAccount")
			End If

			If String.IsNullOrWhiteSpace(liveKey) Then
				Throw New ArgumentNullException("liveKey")
			End If

			If String.IsNullOrWhiteSpace(liveScope) Then
				Throw New ArgumentNullException("liveScope")
			End If

			If liveRestEndpoint Is Nothing Then
				Throw New ArgumentNullException("liveRestEndpoint")
			End If

			If liveAcsEndpoint Is Nothing Then
				Throw New ArgumentNullException("liveAcsEndpoint")
			End If

			Me.LiveRestEndpoint = liveRestEndpoint
			Me.LiveAcsEndpoint = liveAcsEndpoint
			Me.LiveAccount = liveAccount
			Me.LiveKey = liveKey
			Me.LiveScope = liveScope

			Me.LiveServices = Nothing

			If Me.LiveServices Is Nothing Then
				' Account name
					' Account key
				Me.LiveServices = New CloudMediaContext(Me.LiveAccount, Me.LiveKey)
			End If
		End Sub

		''' <summary>
		''' Implement disposable interface.
		''' </summary>
        Public Sub Dispose() Implements IDisposable.Dispose
        End Sub
	End Class
End Namespace
