
' ----------------------------------------------------------------------
' <copyright file="MediaExtensions.cs" company="Microsoft">
' Copyright © Microsoft Corporation.  All rights reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
' FITNESS FOR A PARTICULAR PURPOSE.
' </copyright>
'  <notes>
'                  \\|//
'                  (o o)
' --------------ooO-(_)-Ooo--------------------
'  Authors: Developer Product Evangelist Team (greg.prentice)
'  License: Apache License Version 2.0, January 2004
'                       Oooo
' --------------oooO----(  )-------------------
'               (  )    (_/
'                \_)
'  </notes>
' -----------------------------------------------------------------------

Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports Microsoft.WindowsAzure.MediaServices.Client
Namespace LiveMediaManager

	''' <summary>
	''' Defines methods for interacting with the Azure Media services.
	''' </summary>
    Public Module MediaExtensions


        ''' <summary>
        ''' Looks for the existence of a file that ends with the ".ism" extension and
        ''' represents the manifest file associated with adaptive streaming.
        ''' </summary>
        ''' <param name="asset">the reference to the asset</param>
        ''' <returns>a reference to the first occurrence or null</returns>
        Public Function FindManifest(asset As IAsset) As IAssetFile
            If asset Is Nothing Then
                Throw New ArgumentNullException("asset")
            End If

            ' Get a reference to the streaming manifest file from the  
            ' collection of files in the asset. 
            Dim theManifest = From f In asset.AssetFiles Where f.Name.EndsWith(".ism") Select f

            ' Cast the reference to a true IAssetFile type. 
            Return theManifest.FirstOrDefault()
        End Function

        ''' <summary>
        '''  Returns an origin servers by name.
        ''' </summary>
        ''' <param name="liveServices">the extension methods target object</param>
        ''' <param name="name">the name of the origin server</param>
        ''' <returns>a reference to an origin server or null if not found</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function FindStreamingEndpoint(liveServices As CloudMediaContext, name As String) As IStreamingEndpoint
            If String.IsNullOrEmpty(name) Then
                Throw New ArgumentNullException("name")
            End If

            Dim origin As IStreamingEndpoint = (From o In liveServices.StreamingEndpoints Where o.Name = name).First()

            Return origin
        End Function

        <System.Runtime.CompilerServices.Extension> _
        Public Function FindFirstStreamingEndpoint(liveServices As CloudMediaContext) As IStreamingEndpoint
            Dim origin As IStreamingEndpoint = (From o In liveServices.StreamingEndpoints).First()
            Return origin
        End Function

        ''' <summary>
        ''' Retruns the channel by name.
        ''' </summary>
        ''' <param name="liveServices">>the extension methods target object</param>
        ''' <param name="name">name of the channel</param>
        ''' <returns>a reference to the channel or null if not found</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function FindChannel(liveServices As CloudMediaContext, name As String) As IChannel
            If String.IsNullOrEmpty(name) Then
                Throw New ArgumentNullException("name")
            End If

            Dim channel As IChannel = (From x In liveServices.Channels Where x.Name = name).FirstOrDefault()
            Return channel
        End Function
        ''' <summary>
        ''' Locates the asset associated with this program.
        ''' </summary>
        ''' <param name="liveServices">the extension methods target object</param>
        ''' <param name="program">program that contains the asset</param>
        ''' <returns>a reference to the asset or null if not found.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function FindAsset(liveServices As CloudMediaContext, program As IProgram) As IAsset
            If program Is Nothing Then
                Throw New ArgumentNullException("program")
            End If

            Dim asset As IAsset = (From x In liveServices.Assets Where x.Id = program.AssetId).FirstOrDefault()
            Return asset
        End Function

        ''' <summary>
        ''' Creates a locator and returns 
        ''' </summary>
        ''' <param name="liveServices">the extension methods target object</param>
        ''' <param name="asset">a reference to the asset</param>
        ''' <param name="duration">a time span that is equal to or greater than 5 minutes</param>
        ''' <returns>a list of URI's to Smooth, HLS, and DASH streaming media</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Sub CreateLocators(liveServices As CloudMediaContext, asset As IAsset, duration As TimeSpan)
            If asset Is Nothing Then
                Throw New ArgumentNullException("asseet")
            End If

            If IsNothing(duration) OrElse duration.TotalMinutes < 5 Then
                Throw New ArgumentOutOfRangeException("numberOfDays")
            End If

            Dim originLocator As ILocator

            If asset.Locators.Count() = 0 Then
                ' Create a readonly access policy. 
                Dim policy As IAccessPolicy = liveServices.AccessPolicies.Create("Streaming policy", duration, AccessPermissions.Read)

                ' Create a locator to the streaming content on an origin. 
                originLocator = liveServices.Locators.CreateLocator(LocatorType.OnDemandOrigin, asset, policy, DateTime.UtcNow.AddMinutes(-5))
            End If
        End Sub

        ''' <summary>
        ''' Creates a list of URI's for accessing specific formats. 
        ''' </summary>
        ''' <param name="liveServices">the extension methods target object</param>
        ''' <param name="asset">a reference to the asset</param>
        ''' <returns>a list of URI's to Smooth, HLS, and DASH streaming media</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function CreateStreamingURIs(liveServices As CloudMediaContext, asset As IAsset) As IDictionary(Of VideoFormat, String)
            If asset Is Nothing Then
                Throw New ArgumentNullException("asseet")
            End If

            Dim manifestFile As IAssetFile = MediaExtensions.FindManifest(asset)

            If manifestFile Is Nothing Then
                Throw New ArgumentException("No manifest file exists", "assetToStream")
            End If

            Dim originLocator As ILocator
            Dim uris As New Dictionary(Of VideoFormat, String)()

            If asset.Locators.Count() <> 0 Then
                originLocator = asset.Locators.First()
                ' Create locator strings for Smooth, HLS, DASH
                uris.Add(VideoFormat.SmoothStreaming, originLocator.Path + manifestFile.Name + "/manifest")
                uris.Add(VideoFormat.HttpLiveStreaming, String.Format("{0}{1}", uris(0), "(format=m3u8-aapl)"))
                uris.Add(VideoFormat.DynamicAdaptiveStreaming, String.Format("{0}{1}", uris(0), "(format=mpd-time-csf)"))


                Dim x As Uri = asset.GetHlsUri()
                x = asset.GetMpegDashUri()

                x = asset.GetSmoothStreamingUri()
            End If

            Return uris
        End Function

    End Module
End Namespace
